//
// HttpServletResponse.java
//

package nyangau.se;

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
//...e

public class HttpServletResponse
  extends ServletResponse
  implements jakarta.servlet.http.HttpServletResponse
  {
  protected HttpServletRequest req;
  protected int status = jakarta.servlet.http.HttpServletResponse.SC_OK;
  protected Dictionary<String,Vector<String>> headers = new Hashtable<String,Vector<String>>();
  // interface jakarta.servlet.http.HttpServletResponse
//...scontainsHeader:2:
public boolean containsHeader(String h)
  {
  return headers.get(h) != null;
  }
//...e
//...saddHeader:2:
public void addHeader(String h, String v)
  {
  Vector <String>vec;
  if ( (vec = headers.get(h)) == null )
    {
    vec = new Vector<String>();
    headers.put(h, vec);
    }
  vec.add(v);
  }
//...e
//...ssetHeader:2:
public void setHeader(String h, String v)
  {
  headers.remove(h);
  addHeader(h, v);
  }
//...e
//...saddIntHeader:2:
public void addIntHeader(String h, int v)
  {
  addHeader(h, Integer.toString(v));
  }
//...e
//...ssetIntHeader:2:
public void setIntHeader(String h, int v)
  {
  setHeader(h, Integer.toString(v));
  }
//...e
//...sfmt7231:2:
// RFC 7231 gives this as an example of the preferred date format
//   Sun, 06 Nov 1994 08:49:37 GMT
// and calls it the IMF-fixdate.
// When I format on new Date(0), I find that the hour comes back as 01.
// This is because GMT+1 was (rather unusually) in force at that time.
// However, the timezone still reports as GMT, rather than BST.
// So best to force the formatter in GMT prior to using it.

private static final String fmt7231 = "EEE, dd MMM yyyy HH:mm:ss zzz";
//...e
//...saddDateHeader:2:
public void addDateHeader(String h, long v)
  {
  SimpleDateFormat sdf = new SimpleDateFormat(fmt7231);
  sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
  addHeader(h, sdf.format(new Date(v)));
  }
//...e
//...ssetDateHeader:2:
public void setDateHeader(String h, long v)
  {
  SimpleDateFormat sdf = new SimpleDateFormat(fmt7231);
  sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
  setHeader(h, sdf.format(new Date(v)));
  }
//...e
//...ssetStatus:2:
public void setStatus(int sc)
  {
  status = sc;
  }
//...e
//...ssendError:2:
public void sendError(int sc)
  throws IOException
  {
  if ( isCommitted() )
    throw new IllegalStateException();
  setStatus(sc);
  headers = new Hashtable<String,Vector<String>>();
  setContentType("text/html");
  jakarta.servlet.ServletOutputStream os = getOutputStream();
  os.println("<H1>Error "+sc+"</H1>");
  }
//...e
//...ssendError:2:
public void sendError(int sc, String message)
  throws IOException
  {
  if ( isCommitted() )
    throw new IllegalStateException();
  setStatus(sc);
  headers = new Hashtable<String,Vector<String>>();
  setContentType("text/html");
  jakarta.servlet.ServletOutputStream os = getOutputStream();
  os.println("<H1>Error "+sc+"</H1>");
  os.println(message);
  }
//...e
//...ssendRedirect:2:
// Support for relative URLs introduced at Servlet 2.2

// Note, the req.getHeader("Host") is a security issue.

public void sendRedirect(String url)
  throws IOException
  {
  if ( isCommitted() )
    throw new IllegalStateException();
  setStatus(SC_MOVED_TEMPORARILY);
  headers = new Hashtable<String,Vector<String>>();
  if ( url.startsWith("http://") || url.startsWith("https://") )
    // Already an absolute URL
    ;
  else if ( url.startsWith("/") )
    // URL is incomplete as it has no http://host:port
    // Not sure this kind of URL is technically considered "relative"
    // We support it here
    url = req.getScheme()+"://" + req.getHeader("Host") + url;
  else
    // Relative URL, construct an absolute URL from it
    url = req.getScheme()+"://" + req.getHeader("Host") + req.getRequestURI() + "/" + url;
  setHeader("Location", url);
  setContentType("text/html");
  jakarta.servlet.ServletOutputStream os = getOutputStream();
  os.println("<P>Redirecting...");
  }
//...e
//...sencodeURL:2:
public String encodeURL(String url)
  {
  // No session to encode in the URL
  return url;
  } 
//...e
//...sencodeRedirectURL:2:
public String encodeRedirectURL(String url)
  {
  // No session to encode in the URL
  return url;
  } 
//...e
  // overriding jakarta.servlet.ServletResponse
//...ssetContentType:2:
public void setContentType(String type)
  {
  setHeader("Content-Type", type);
  }
//...e
//...ssetContentLength:2:
public void setContentLength(int length)
  {
  setIntHeader("Content-Length", length);
  }
//...e
  // overriding ServletResponse
//...sensureCommitted:2:
protected void ensureCommitted()
  throws IOException
  {
  if ( ! isCommitted() )
    {
    super.ensureCommitted();
    if ( req.getRespHdrs() )
      {
      os.println("HTTP/1.1 "+status+" OK");
      for ( Enumeration<String> e = headers.keys(); e.hasMoreElements(); )
        {
        String h = e.nextElement();
        Vector<String> vec = headers.get(h);
        for ( int i = 0; i < vec.size(); i++ )
          os.println(h+": "+vec.elementAt(i));
        }
      os.println();
      }
    }
  }
//...e
  // other
//...sHttpServletResponse:2:
public HttpServletResponse(
  ServletEngine se,
  HttpServletRequest request,
  ServletOutputStream servletOutputStream
  )
  {
  super(servletOutputStream);
  req = request;
  String server = se.getServer();
  if ( ! server.equals("") )
    setHeader("Server", server);
  }
//...e
  }
