//
// ServletContext.java
//
// Provide access to context-wide init parameters.
// In this case, this is the whole application server.
//
// Provide mapping from filename to MIME type.
// Look in the context-wide init parameters for mime.EXT=TYPE.
// Failing this, resort to looking in our hard-coded internal values.
//

package nyangau.se;

import java.net.*;
import java.io.*;
import java.util.*;

public class ServletContext implements jakarta.servlet.ServletContext
  {
  Properties properties;
  protected Dictionary<String,String> mimes = new Hashtable<String,String>();
  protected String docRoot;
  // interface jakarta.servlet.ServletContext
//...sgetInitParameter:2:
public String getInitParameter(String n)
  {
  return properties.getProperty(n);
  }
//...e
//...sgetInitParameterNames:2:
public Enumeration<String> getInitParameterNames()
  {
  // .propertyNames() predates generics and is of type Enumeration<?>
  // We can't assign it directly
  // We can't cleanly cast it
  // We must cast and suppress
  @SuppressWarnings("unchecked")
  Enumeration<String> e = (Enumeration<String>) properties.propertyNames();
  return e;
  }
//...e
//...sgetMimeType:2:
public String getMimeType(String fn)
  {
  String bfn = fn.substring(fn.lastIndexOf('/')+1);
  int dot;
  if ( (dot = bfn.lastIndexOf('.')) != -1 )
    {
    String ext = bfn.substring(dot+1);
    String type = null;
    if ( (type = properties.getProperty("mime."+ext)) != null )
      return type;
    if ( (type = mimes.get(ext)) != null )
      return type;
    }
  return null;
  }
//...e
//...sgetContext:2:
public jakarta.servlet.ServletContext getContext(String uripath)
  {
  // We've only got one ServletContext, as we host one Web Application
  return this;
  }
//...e
//...sgetResourceAsStream:2:
// path must start with / and is interpreted relative to context root.

public InputStream getResourceAsStream(String path)
  {
  if ( docRoot != null && path.startsWith("/") )
    return Streamer.getResourceAsStream(docRoot, path);
  return null;
  }
//...e
  // other
//...ssetupMimeMappings:2:
protected void setupMimeMappings()
  {
  mimes.put("doc",    "application/msword");
  mimes.put("xls",    "application/vnd.ms-excel");
  mimes.put("ppt",    "application/vnd.ms-powerpoint");
  mimes.put("mpp",    "application/vnd.ms-project");
  mimes.put("docx",   "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
  mimes.put("xlsx",   "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
  mimes.put("pptx",   "application/vnd.openxmlformats-officedocument.presentationml.presentation");
  mimes.put("bin",    "application/octet-stream");
  mimes.put("oda",    "application/oda");
  mimes.put("pdf",    "application/pdf");
  mimes.put("eps",    "application/postscript");
  mimes.put("ps" ,    "application/postscript");
  mimes.put("rtf",    "application/rtf");
  mimes.put("mif",    "application/x-mif");
  mimes.put("csh",    "application/x-csh");
  mimes.put("dvi",    "application/x-dvi");
  mimes.put("hdf",    "application/x-hdf");
  mimes.put("latex",  "application/x-latex");
  mimes.put("nc",     "application/x-netcdf");
  mimes.put("cdf",    "application/x-netcdf");
  mimes.put("sh",     "application/x-sh");
  mimes.put("tcl",    "application/x-tcl");
  mimes.put("tex",    "application/x-tex");
  mimes.put("texinfo","application/x-texinfo");
  mimes.put("texi",   "application/x-texinfo");
  mimes.put("t",      "application/x-troff");
  mimes.put("tr",     "application/x-troff");
  mimes.put("roff",   "application/x-troff");
  mimes.put("man",    "application/x-troff-man");
  mimes.put("me",     "application/x-troff-me");
  mimes.put("ms",     "application/x-troff-ms");
  mimes.put("src",    "application/x-wais-source");
  mimes.put("zip",    "application/x-zip-compressed");
  mimes.put("bcpio",  "application/x-bcpio");
  mimes.put("cpio",   "application/x-cpio");
  mimes.put("gtar",   "application/x-gtar");
  mimes.put("shar",   "application/x-shar");
  mimes.put("tar",    "application/x-tar");
  mimes.put("au",     "audio/basic");
  mimes.put("snd",    "audio/basic");
  mimes.put("aif",    "audio/x-aiff");
  mimes.put("aiff",   "audio/x-aiff");
  mimes.put("aifc",   "audio/x-aiff");
  mimes.put("wav",    "audio/x-wav");
  mimes.put("gif",    "image/gif");
  mimes.put("png",    "image/png");
  mimes.put("jpg",    "image/jpeg");
  mimes.put("jpe",    "image/jpeg");
  mimes.put("jpeg",   "image/jpeg");
  mimes.put("tif",    "image/tiff");
  mimes.put("tiff",   "image/tiff");
  mimes.put("ras",    "image/x-cmu-raster");
  mimes.put("pnm",    "image/x-portable-anymap");
  mimes.put("pbm",    "image/x-portable-bitmap");
  mimes.put("pgm",    "image/x-portable-graymap");
  mimes.put("ppm",    "image/x-portable-pixmap");
  mimes.put("rgb",    "image/x-rgb");
  mimes.put("xbm",    "image/x-xbitmap");
  mimes.put("xpm",    "image/x-xpixmap");
  mimes.put("xwd",    "image/x-xwindowdump");
  mimes.put("htm",    "text/html");
  mimes.put("html",   "text/html");
  mimes.put("xml",    "text/xml");
  mimes.put("xsl",    "text/xml");
  mimes.put("xslt",   "text/xml");
  mimes.put("xsd",    "text/xml");
  mimes.put("sgml",   "text/x-sgml");
  mimes.put("sgm",    "text/x-sgml");
  mimes.put("txt",    "text/plain");
  mimes.put("text",   "text/plain");
  mimes.put("rtx",    "text/richtext");
  mimes.put("csv",    "text/csv");
  mimes.put("tsv",    "text/tab-separated-values");
  mimes.put("mpg",    "video/mpeg");
  mimes.put("mpe",    "video/mpeg");
  mimes.put("mpeg",   "video/mpeg");
  mimes.put("qt",     "video/quicktime");
  mimes.put("mov",    "video/quicktime");
  mimes.put("avi",    "video/x-msvideo");
  mimes.put("movie",  "video/x-sgi-movie");
  mimes.put("svg",    "image/svg+xml");
  mimes.put("css",    "text/css");
  mimes.put("js",     "text/javascript");
  }
//...e
//...sServletContext:2:
public ServletContext(Properties p, String dr)
  {
  setupMimeMappings();
  properties = p;
  docRoot = dr;
  }
public ServletContext(String propfn, String dr)
  throws IOException
  {
  setupMimeMappings();
  properties = new Properties();
  try
    (
    InputStream is = Streamer.getPropertyStream(propfn);
    )
    {
    properties.load(is);
    }
  docRoot = dr;
  }
//...e
  }
